Recharge = {
	version				= "2.1",
    author				= "XanDDemoX, modified by Baertram",
    name				= "Auto Recharge",
    displayName			= "Auto Recharge",
    website				= "http://www.esoui.com/downloads/info1091-AutoRecharge.html#info",
	savedVarsName		= "AutoRecharge_SavedVariables",
    savedVarsVersion	= 2.1, --Changing this will reset all SavedVariables!
}
Recharge.preChatTextRed = "|cDD2222AutoRecharge|r "

local _repairSlots		= {EQUIP_SLOT_OFF_HAND,EQUIP_SLOT_BACKUP_OFF, EQUIP_SLOT_HEAD, EQUIP_SLOT_SHOULDERS, EQUIP_SLOT_CHEST,EQUIP_SLOT_WAIST, EQUIP_SLOT_LEGS,EQUIP_SLOT_HAND, EQUIP_SLOT_FEET}
local _rechargeSlots	= {EQUIP_SLOT_MAIN_HAND,EQUIP_SLOT_OFF_HAND,EQUIP_SLOT_BACKUP_MAIN,EQUIP_SLOT_BACKUP_OFF}

local _slotText = {
	[EQUIP_SLOT_MAIN_HAND] 		= GetString(SI_EQUIPTYPE14),
	[EQUIP_SLOT_OFF_HAND] 		= GetString(SI_EQUIPTYPE7),
	[EQUIP_SLOT_BACKUP_MAIN] 	= GetString(SI_EQUIPSLOT20),
	[EQUIP_SLOT_BACKUP_OFF] 	= GetString(SI_EQUIPSLOT21),
	[EQUIP_SLOT_HEAD] 			= GetString(SI_EQUIPTYPE1),
	[EQUIP_SLOT_SHOULDERS] 		= GetString(SI_EQUIPTYPE4),
	[EQUIP_SLOT_CHEST] 			= GetString(SI_EQUIPTYPE3),
	[EQUIP_SLOT_WAIST] 			= GetString(SI_EQUIPTYPE8),
	[EQUIP_SLOT_LEGS] 			= GetString(SI_EQUIPTYPE9),
	[EQUIP_SLOT_HAND] 			= GetString(SI_EQUIPTYPE13),
	[EQUIP_SLOT_FEET] 			= GetString(SI_EQUIPTYPE10)
 }

local _prefix = "|c22DD22[AutoRecharge]|r "
local _defaultSettings = {
	chargeEnabled						= true,
    minChargePercent					= 0,
    alertSoulGemsEmpty					= false,
    alertSoulGemsSoonEmpty				= false,
    alertSoulGemsSoonEmptyThreshold		= 20,
    alertSoulGemsEmptyOnLogin			= true,

	repairEnabled						= true,
    minConditionPercent					= 0,
    alertRepairKitsEmpty				= false,
    alertRepairKitsSoonEmpty			= false,
    alertRepairKitsSoonEmptyThreshold	= 10,
    alertRepairKitsEmptyOnLogin			= true,

    chatOutput							= true,
    chatOutputSuppressNothingMessages	= true,
}

local _settings = {}

local function round(value,places)
	local s =  10 ^ places
	return math.floor(value * s + 0.5) / s
end

local function trim(str)
	if str == nil or str == "" then return str end 
	return (str:gsub("^%s*(.-)%s*$", "%1"))
end 

local function GetEquipSlotText(slot)
	return _slotText[slot]
end

local function println(...)
	local args = {...}
	for i,v in ipairs(args) do
		args[i] = tostring(v)
	end 
	table.insert(args,1,_prefix)
	d(table.concat(args))
end

local function showAlertMessage(alertType, value)
	if not alertType then return false end
    local alertMsg
    local iconText
    if alertType == "repairKitsEmpty" then
    	alertMsg = GetString(SI_ARC_ALERT_REPAIRKITS_EMPTY)
        iconText = zo_iconTextFormat("esoui/art/icons/quest_crate_001.dds", 48, 48, " ")
    elseif alertType == "repairKitsSoonEmpty" then
    	if not value then return end
    	alertMsg = zo_strformat(GetString(SI_ARC_ALERT_REPAIRKITS_SOON_EMPTY), value)
        iconText = zo_iconTextFormat("esoui/art/icons/quest_crate_001.dds", 48, 48, " ")
    elseif alertType == "soulGemsEmpty" then
    	alertMsg = GetString(SI_ARC_ALERT_SOULGEMS_EMPTY)
        iconText = zo_iconTextFormat("esoui/art/icons/soulgem_006_empty.dds", 48, 48, " ")
    elseif alertType == "soulGemsSoonEmpty" then
    	if not value then return end
    	alertMsg = zo_strformat(GetString(SI_ARC_ALERT_SOULGEMS_SOON_EMPTY), value)
        iconText = zo_iconTextFormat("esoui/art/icons/soulgem_006_empty.dds", 48, 48, " ")
    end
    if alertMsg and alertMsg ~= "" then
		CENTER_SCREEN_ANNOUNCE:AddMessage(EVENT_SKILL_RANK_UPDATE, CSA_EVENT_SMALL_TEXT, nil, iconText .. Recharge.preChatTextRed .. alertMsg)
    end
end

local function checkThresholdOrEmpty(checkType, emptyOrThreshold, amount)
	if checkType == nil then return false end
    if emptyOrThreshold == nil then return false end
    if amount == nil then return false end
    if 		checkType == "repairKits" then
    	if emptyOrThreshold then
            if amount == 0 then
	        	showAlertMessage("repairKitsEmpty")
            end
        else
			if amount < _settings.alertRepairKitsSoonEmptyThreshold then
				showAlertMessage("repairKitsSoonEmpty", amount)
            end
        end
    elseif 	checkType == "soulGems" then
    	if emptyOrThreshold then
            if amount == 0 then
	        	showAlertMessage("soulGemsEmpty")
            end
        else
			if amount < _settings.alertSoulGemsSoonEmptyThreshold then
				showAlertMessage("soulGemsSoonEmpty", amount)
            end
        end
    end
end

local function ChargeEquipped(chatOutput, inCombat)

	chatOutput = chatOutput or false

	local gems = Recharge.Bag.GetSoulGems(BAG_BACKPACK)

	local gemsCount = #gems
	if gemsCount == 0 then
		if chatOutput then println(GetString(SI_ARC_ALERT_SOULGEMS_EMPTY)) end
		if not inCombat and _settings.alertSoulGemsEmpty then
		    checkThresholdOrEmpty("soulGems", true, gemsCount)
        end
		return
    else
        gemsCount = 0
    	for _, soulGemStack in pairs(gems) do
    		gemsCount = gemsCount + soulGemStack.size
        end
	end

	local total = 0

	local str

	for i,slot in ipairs(_rechargeSlots) do

		total = Recharge.Charge.ChargeItem(BAG_WORN,slot,gems,_settings.minChargePercent)

		if total > 0 and chatOutput then
			str = (str or GetString(SI_ARC_CHATOUTPUT_CHARGED))..((str and ", ") or "")..GetEquipSlotText(slot).." ("..tostring(round(total,2)).." %)"
		end

	end

	if str == nil and chatOutput and not _settings.chatOutputSuppressNothingMessages then

		local charge,maxcharge

		for i,slot in ipairs(_rechargeSlots) do

			charge,maxcharge = GetChargeInfoForItem(BAG_WORN,slot)

			if maxcharge > 0 then
				str = (str or GetString(SI_ARC_CHATOUTPUT_CHARGED_NOTHING))..((str and ", ") or "")..GetEquipSlotText(slot).." ("..tostring(round((charge / maxcharge) * 100,2)).." %)"
			end
		end

	end

	if str ~= nil and chatOutput then
		println(str)
	elseif str == nil and chatOutput and not _settings.chatOutputSuppressNothingMessages then
		println(GetString(SI_ARC_CHATOUPUT_NO_CHARGEABLE_WEAPONS))
	end

	if not inCombat and _settings.alertSoulGemsSoonEmpty then
	    checkThresholdOrEmpty("soulGems", false, gemsCount)
	end
end

local function RepairEquipped(chatOutput, inCombat)
	chatOutput = chatOutput or false

	local kits = Recharge.Bag.GetRepairKits(BAG_BACKPACK)
	local kitsCount = #kits
	if kitsCount == 0 then
		if chatOutput then println(GetString(SI_ARC_ALERT_REPAIRKITS_EMPTY)) end
		if not inCombat and _settings.alertRepairKitsEmpty then
		    checkThresholdOrEmpty("repairKits", true, kitsCount)
        end
		return
    else
        kitsCount = 0
    	for _, repairKitsStack in pairs(kits) do
    		kitsCount = kitsCount + repairKitsStack.size
        end
    end

	local total = 0

	local str

	for i,slot in ipairs(_repairSlots) do

		total = Recharge.Repair.RepairItem(BAG_WORN,slot,kits, _settings.minConditionPercent)

		if total > 0 and chatOutput then
			str = (str or GetString(SI_ARC_CHATOUTPUT_REPAIRED))..((str and ", ") or "")..GetEquipSlotText(slot).." ("..tostring(round(total,2)).." %)"
		end

	end

	if str == nil and chatOutput and not _settings.chatOutputSuppressNothingMessages then

		local condition

		for i,slot in ipairs(_repairSlots) do
			condition = GetItemCondition(BAG_WORN,slot)
			str = (str or GetString(SI_ARC_CHATOUTPUT_REPAIRED_NOTHING))..((str and ", ") or "")..GetEquipSlotText(slot).." ("..tostring(round(condition,2)).." %)"
		end
	end

	if str ~= nil and chatOutput then
		println(str)
	elseif str == nil and chatOutput and not _settings.chatOutputSuppressNothingMessages then
		println(GetString(SI_ARC_CHATOUPUT_NO_REPAIRABLE_ARMOR))
	end

	if not inCombat and _settings.alertRepairKitsSoonEmpty  then
	    checkThresholdOrEmpty("repairKits", false, kitsCount)
	end
end

local function IsPlayerDead()
	return IsUnitDead("player")
end

local function ARC_CombatStateChanged(eventCode, inCombat)
	if IsPlayerDead() == true then return end

	if _settings.chargeEnabled == true then
		ChargeEquipped(_settings.chatOutput, inCombat)
	end

	if _settings.repairEnabled == true then
		RepairEquipped(_settings.chatOutput, inCombat)
	end

end

local function TryParseOnOff(str)
	local on = (str == "+" or str == "on")
	local off = (str == "-" or str == "off")
	if on == false and off == false then return nil end
	return on
end

local function TryParsePercent(str)
	local percent = tonumber(str)
	if percent ~= nil and percent >= 0 and percent <= 100 then return (percent / 100) end
	return nil
end

--Build the addon's LAM 2.0 settings menu
--Create the settings panel object of libAddonMenu 2.0
local LAM = LibStub('LibAddonMenu-2.0')
local function BuildAddonMenu()

	local panelData = {
		type 				= 'panel',
		name 				= Recharge.name,
		displayName 		= Recharge.displayName,
		author 				= Recharge.author,
		version 			= Recharge.version,
		registerForRefresh 	= true,
		registerForDefaults = true,
		slashCommand 		= "/recharge",
        website 			= Recharge.website,
	}
	
	LAM:RegisterAddonPanel("Auto Recharge", panelData)

	local optionsTable =
    {	-- BEGIN OF OPTIONS TABLE

--Description of the addon
		{
			type = "description",
			text = GetString(SI_ARC_LAM_ADDON_DESC),
		},
		--Automatic recharging
		{
        	type = "header",
        	name = GetString(SI_ARC_LAM_HEADER_AUTO_CHARGE),
        },
		{
			type = "checkbox",
			name = GetString(SI_ARC_LAM_OPTION_AUTO_RECHARGE),
			tooltip = GetString(SI_ARC_LAM_OPTION_AUTO_RECHARGE_TT),
			getFunc = function() return _settings.chargeEnabled end,
			setFunc = function(value) _settings.chargeEnabled = value end,
            default = _settings.chargeEnabled,
            width="half",
		},
 		{
			type = "slider",
			name = GetString(SI_ARC_LAM_OPTION_AUTO_RECHARGE_MIN_TH),
			tooltip = GetString(SI_ARC_LAM_OPTION_AUTO_RECHARGE_MIN_TH_TT),
			min = 0,
			max = 99,
			getFunc = function() return (_settings.minChargePercent*100) end,
			setFunc = function(percent)
					local percentage = TryParsePercent(percent)
					if percentage ~= nil and percentage >= 0 and percentage < 1 then
						_settings.minChargePercent = percentage
	                    if _settings.chatOutput then println(GetString(SI_ARC_CHATOUPUT_MIN_CHARGE),tostring(percent),"%") end
                    end
 				end,
            width="half",
			default = (_settings.minChargePercent*100),
            disabled = function() return not _settings.chargeEnabled end,
		},
		--Automatic repairing
		{
        	type = "header",
        	name = GetString(SI_ARC_LAM_HEADER_AUTO_REPAIR),
        },
		{
			type = "checkbox",
			name = GetString(SI_ARC_LAM_OPTION_AUTO_REPAIR),
			tooltip = GetString(SI_ARC_LAM_OPTION_AUTO_REPAIR_TT),
			getFunc = function() return _settings.repairEnabled end,
			setFunc = function(value) _settings.repairEnabled = value end,
            default = _settings.repairEnabled,
            width="half",
		},
 		{
			type = "slider",
			name = GetString(SI_ARC_LAM_OPTION_AUTO_REPAIR_MIN_TH),
			tooltip = GetString(SI_ARC_LAM_OPTION_AUTO_REPAIR_MIN_TH_TT),
			min = 0,
			max = 99,
			getFunc = function() return (_settings.minConditionPercent*100) end,
			setFunc = function(percent)
					local percentage = TryParsePercent(percent)
					if percentage ~= nil and percentage >= 0 and percentage < 1 then
						_settings.minConditionPercent = percentage
						if _settings.chatOutput then println(GetString(SI_ARC_CHATOUPUT_MIN_CONDITION),tostring(percent),"%") end
                    end
 				end,
            width="half",
			default = (_settings.minConditionPercent*100),
            disabled = function() return not _settings.repairEnabled end,
		},
		--Chat output
		{
        	type = "header",
        	name = GetString(SI_ARC_LAM_HEADER_CHATOUTPUT),
        },
		{
			type = "checkbox",
			name = GetString(SI_ARC_LAM_OPTION_SHOW_CHAT_MSG),
			tooltip = GetString(SI_ARC_LAM_OPTION_SHOW_CHAT_MSG_TT),
			getFunc = function() return _settings.chatOutput end,
			setFunc = function(value) _settings.chatOutput = value end,
            default = _settings.chatOutput,
            width="half",
		},
		{
			type = "checkbox",
			name = GetString(SI_ARC_LAM_OPTION_SUPPRESS_CHAT_MSG_NOTHING),
			tooltip = GetString(SI_ARC_LAM_OPTION_SUPPRESS_CHAT_MSG_NOTHING_TT),
			getFunc = function() return _settings.chatOutputSuppressNothingMessages end,
			setFunc = function(value) _settings.chatOutputSuppressNothingMessages = value end,
            default = _settings.chatOutputSuppressNothingMessages,
            disabled = function() return not _settings.chatOutput end,
            width="half",
		},
		--Onscreen messages
		{
        	type = "header",
        	name = GetString(SI_ARC_LAM_HEADER_ALERT),
        },
		{
			type = "checkbox",
			name = GetString(SI_ARC_LAM_OPTION_ALERT_EMPTY_REPAIRKIT),
			tooltip = GetString(SI_ARC_LAM_OPTION_ALERT_EMPTY_REPAIRKIT_TT),
			getFunc = function() return _settings.alertRepairKitsEmpty end,
			setFunc = function(value) _settings.alertRepairKitsEmpty = value end,
            default = _settings.alertRepairKitsEmpty,
            width="full",
		},
		{
			type = "checkbox",
			name = GetString(SI_ARC_LAM_OPTION_ALERT_SOON_EMPTY_REPAIRKIT),
			tooltip = GetString(SI_ARC_LAM_OPTION_ALERT_SOON_EMPTY_REPAIRKIT_TT),
			getFunc = function() return _settings.alertRepairKitsSoonEmpty end,
			setFunc = function(value) _settings.alertRepairKitsSoonEmpty = value end,
            default = _settings.alertRepairKitsSoonEmpty,
            width="half",
		},
 		{
			type = "slider",
			name = GetString(SI_ARC_LAM_OPTION_ALERT_EMPTY_REPAIRKIT_TH),
			tooltip = GetString(SI_ARC_LAM_OPTION_ALERT_EMPTY_REPAIRKIT_TH_TT),
			min = 1,
			max = 100,
			getFunc = function() return _settings.alertRepairKitsSoonEmptyThreshold end,
			setFunc = function(value)
						_settings.alertRepairKitsSoonEmptyThreshold = value
						if _settings.chatOutput then println(GetString(SI_ARC_CHATOUPUT_REPAIRKITS_EMPTY_THRESHOLD),tostring(value)) end
 				end,
            width="half",
			default = _settings.alertRepairKitsSoonEmptyThreshold,
            disabled = function() return not _settings.alertRepairKitsSoonEmpty end,
		},
		{
			type = "checkbox",
			name = GetString(SI_ARC_LAM_OPTION_ALERT_EMPTY_REPAIRKIT_LOGIN),
			tooltip = GetString(SI_ARC_LAM_OPTION_ALERT_EMPTY_REPAIRKIT_LOGIN_TT),
			getFunc = function() return _settings.alertRepairKitsEmptyOnLogin end,
			setFunc = function(value) _settings.alertRepairKitsEmptyOnLogin = value end,
            default = _settings.alertRepairKitsEmptyOnLogin,
            width="full",
		},

		{
			type = "checkbox",
			name = GetString(SI_ARC_LAM_OPTION_ALERT_EMPTY_SOULGEM),
			tooltip = GetString(SI_ARC_LAM_OPTION_ALERT_EMPTY_SOULGEM_TT),
			getFunc = function() return _settings.alertSoulGemsEmpty end,
			setFunc = function(value) _settings.alertSoulGemsEmpty = value end,
            default = _settings.alertSoulGemsEmpty,
            width="full",
		},
		{
			type = "checkbox",
			name = GetString(SI_ARC_LAM_OPTION_ALERT_SOON_EMPTY_SOULGEM),
			tooltip = GetString(SI_ARC_LAM_OPTION_ALERT_SOON_EMPTY_SOULGEM_TT),
			getFunc = function() return _settings.alertSoulGemsSoonEmpty end,
			setFunc = function(value) _settings.alertSoulGemsSoonEmpty = value end,
            default = _settings.alertSoulGemsSoonEmpty,
            width="half",
		},
 		{
			type = "slider",
			name = GetString(SI_ARC_LAM_OPTION_ALERT_EMPTY_SOULGEM_TH),
			tooltip = GetString(SI_SI_ARC_LAM_OPTION_ALERT_EMPTY_SOULGEM_TH_TT),
			min = 1,
			max = 100,
			getFunc = function() return _settings.alertSoulGemsSoonEmptyThreshold end,
			setFunc = function(value)
						_settings.alertSoulGemsSoonEmptyThreshold = value
						if _settings.chatOutput then println(GetString(SI_ARC_CHATOUPUT_SOULGEMS_EMPTY_THRESHOLD),tostring(value)) end
 				end,
            width="half",
			default = _settings.alertSoulGemsSoonEmptyThreshold,
            disabled = function() return not _settings.alertSoulGemsSoonEmpty end,
		},
		{
			type = "checkbox",
			name = GetString(SI_ARC_LAM_OPTION_ALERT_EMPTY_SOULGEM_LOGIN),
			tooltip = GetString(SI_ARC_LAM_OPTION_ALERT_EMPTY_SOULGEM_LOGIN_TT),
			getFunc = function() return _settings.alertSoulGemsEmptyOnLogin end,
			setFunc = function(value) _settings.alertSoulGemsEmptyOnLogin = value end,
            default = _settings.alertSoulGemsEmptyOnLogin,
            width="full",
		},

    } -- END OF OPTIONS TABLE
	LAM:RegisterOptionControls("Auto Recharge", optionsTable)

end

local function ARC_Initialise()
	EVENT_MANAGER:RegisterForEvent(Recharge.name,EVENT_PLAYER_COMBAT_STATE,ARC_CombatStateChanged)
		
	SLASH_COMMANDS["/arc"] = function(arg)
		arg = trim(arg)
		if arg == nil or arg == "" then
			if IsPlayerDead() == true then return end
			ChargeEquipped()
		else
			local percent = TryParsePercent(arg)

			if percent ~= nil and percent >= 0 and percent < 1 then
				_settings.minChargePercent = percent
				println(GetString(SI_ARC_CHATOUPUT_MIN_CHARGE),tostring(percent * 100),"%")
			elseif percent ~= nil then
				println(zo_strformat(GetString(SI_ARC_CHATOUPUT_INVALID_PERCENTAGE), (percent * 100)))
			else
				 local enabled = TryParseOnOff(arg)
				 if enabled ~= nil then
					_settings.chargeEnabled = enabled
					println(GetString(SI_ARC_CHATOUPUT_CHARGE), ((_settings.chargeEnabled and GetString(SI_ARC_CHATOUPUT_SETTINGS_ENABLED)) or GetString(SI_ARC_CHATOUPUT_SETTINGS_DISABLED)))
				 end
			end
		end

	end

	SLASH_COMMANDS["/arp"] = function(arg)
		arg = trim(arg)
		if arg == nil or arg == "" then
			if IsPlayerDead() == true then return end
			RepairEquipped()
		else
			local percent = TryParsePercent(arg)

			if percent ~= nil and percent >= 0 and percent < 1 then
				_settings.minConditionPercent = percent
				println(GetString(SI_ARC_CHATOUPUT_MIN_CONDITION),tostring(percent * 100),"%")
			elseif percent ~= nil then
				println(zo_strformat(GetString(SI_ARC_CHATOUPUT_INVALID_PERCENTAGE), (percent * 100)))
			else
				local enabled = TryParseOnOff(arg)
				if enabled ~= nil then
					_settings.repairEnabled = enabled
					println(GetString(SI_ARC_CHATOUPUT_REPAIR),((_settings.repairEnabled and GetString(SI_ARC_CHATOUPUT_SETTINGS_ENABLED)) or GetString(SI_ARC_CHATOUPUT_SETTINGS_DISABLED)))
				end
			end

		end
	end

end

local function ARC_Player_Activated(...)
	EVENT_MANAGER:UnregisterForEvent(Recharge.name, EVENT_PLAYER_ACTIVATED)

    if _settings.alertSoulGemsEmptyOnLogin then
		local gems = Recharge.Bag.GetSoulGems(BAG_BACKPACK)
		local gemsCount = #gems
		if gemsCount == 0 then
		    checkThresholdOrEmpty("soulGems", true, gemsCount)
	    else
	        gemsCount = 0
	    	for _, soulGemStack in pairs(gems) do
	    		gemsCount = gemsCount + soulGemStack.size
	        end
		    checkThresholdOrEmpty("soulGems", false, gemsCount)
		end
    end
    if _settings.alertRepairKitsEmptyOnLogin then
		local kits = Recharge.Bag.GetRepairKits(BAG_BACKPACK)
		local kitsCount = #kits
		if kitsCount == 0 then
		    checkThresholdOrEmpty("repairKits", true, kitsCount)
	    else
	        kitsCount = 0
	    	for _, repairKitsStack in pairs(kits) do
	    		kitsCount = kitsCount + repairKitsStack.size
	        end
		    checkThresholdOrEmpty("repairKits", false, kitsCount)
	    end
    end
end

local function ARC_Loaded(eventCode, addOnName)
	if(addOnName ~= "Recharge") then
        return
    end
	--Unregister this event again so it isn't fired again after this addon has beend recognized
    EVENT_MANAGER:UnregisterForEvent(Recharge.name, EVENT_ADD_ON_LOADED)
    
	_settings = ZO_SavedVars:NewCharacterIdSettings(Recharge.savedVarsName, Recharge.savedVarsVersion, "Settings", _defaultSettings, nil)

	BuildAddonMenu()

	EVENT_MANAGER:RegisterForEvent(Recharge.name, EVENT_PLAYER_ACTIVATED, ARC_Player_Activated)

	ARC_Initialise()
end

EVENT_MANAGER:RegisterForEvent(Recharge.name, EVENT_ADD_ON_LOADED, ARC_Loaded)
